<?php

namespace App\Services\History;

use App\Models\Enums\ActionObjectType;
use App\Services\History\Detector\ProjectDetector;
use Illuminate\Database\Eloquent\Model;

class ModelChangeDetector
{
    public static function detector(ActionObjectType $objectType, Model $model): array
    {
        $detector = match ($objectType) {
            ActionObjectType::PROJECT => ProjectDetector::class,
            default => null
        };

        if (! $detector) {
            return [];
        }

        $fields = call_user_func([$detector, "fields"]);
        $diffFields = call_user_func([$detector, "diffFields"]);

        if (!$fields) {
            return [];
        }

        $items = [];
        foreach ($fields as $field) {
            if (! $model->isDirty($field)) {
                continue;
            }

            $items[] = [
                'old' => $model->getOriginal($field),
                'new' => $model->$field,
                'diff' => in_array($field, $diffFields) ? text_diff($model->getOriginal($field), $model->$field) : null,
            ];
        }

        return $items;
    }
}
